@@ -1,5 +1,5 @@
 /*
- * $Id: request.c,v 1.22 2002/01/30 08:42:34 shugo Exp $
+ * $Id: request.c,v 1.28.2.6 2003/02/07 03:48:38 shugo Exp $
  * Copyright (C) 2000  ZetaBITS, Inc.
  * Copyright (C) 2000  Information-technology Promotion Agency, Japan
  * Copyright (C) 2001  Shugo Maeda <shugo@modruby.net>
@@ -27,17 +27,11 @@
  * SUCH DAMAGE.
  */
 
-#include "httpd.h"
-#include "http_config.h"
-#include "http_core.h"
-#include "http_log.h"
-#include "http_main.h"
-#include "http_protocol.h"
-#include "util_script.h"
-#include "multithread.h"
+#include <sys/types.h>
+#include <sys/stat.h>
 
-#include "ruby.h"
-#include "version.h"
+/* for core_module */
+#define CORE_PRIVATE
 
 #include "mod_ruby.h"
 #include "apachelib.h"
@@ -72,9 +66,11 @@ typedef struct request_data {
 static VALUE fname(VALUE self, VALUE val) \
 { \
     request_data *data; \
+    Check_Type(val, T_STRING); \
     data = get_request_data(self); \
     data->request->member = \
-	ap_pstrdup(data->request->pool, STR2CSTR(val)); \
+	ap_pstrndup(data->request->pool, \
+		    RSTRING(val)->ptr, RSTRING(val)->len); \
     return val; \
 }
 
@@ -99,19 +95,20 @@ static void request_mark(request_data *data)
     rb_gc_mark(data->exception);
 }
 
-static void cleanup_request_object(void *data)
+static APR_CLEANUP_RETURN_TYPE cleanup_request_object(void *data)
 {
     request_rec *r = (request_rec *) data;
     VALUE reqobj;
 
     reqobj = (VALUE) ap_get_module_config(r->request_config, &ruby_module);
-    if (reqobj == 0) return;
+    if (reqobj == 0) APR_CLEANUP_RETURN_SUCCESS();
     if (TYPE(reqobj) == T_DATA) {
 	free(RDATA(reqobj)->data);
 	RDATA(reqobj)->data = NULL;
     }
     ap_set_module_config(r->request_config, &ruby_module, 0);
     rb_apache_unregister_object(reqobj);
+    APR_CLEANUP_RETURN_SUCCESS();
 }
 
 static VALUE apache_request_new(request_rec *r)
@@ -480,6 +477,19 @@ static VALUE request_prev(VALUE self)
     return rb_get_request_object(data->request->prev);
 }
 
+static VALUE request_last(VALUE self)
+{
+    request_data *data;
+    request_rec *last;
+
+    data = get_request_data(self);
+
+    for (last = data->request; last->next; last = last->next)
+        continue;
+
+    return rb_get_request_object(last);
+}
+
 static VALUE request_main(VALUE self)
 {
     request_data *data;
@@ -504,16 +514,25 @@ REQUEST_STRING_ATTR_WRITER(request_set_status_line, status_line);
 REQUEST_STRING_ATTR_READER(request_the_request, the_request);
 REQUEST_STRING_ATTR_READER(request_request_method, method);
 REQUEST_INT_ATTR_READER(request_method_number, method_number);
-REQUEST_STRING_ATTR_READER(request_args, args);
+REQUEST_INT_ATTR_READER(request_get_allowed, allowed);
+REQUEST_INT_ATTR_WRITER(request_set_allowed, allowed);
+REQUEST_STRING_ATTR_READER(request_get_args, args);
+REQUEST_STRING_ATTR_WRITER(request_set_args, args);
 REQUEST_STRING_ATTR_READER(request_get_content_type, content_type);
 REQUEST_STRING_ATTR_READER(request_get_content_encoding, content_encoding);
+REQUEST_STRING_ATTR_READER(request_get_dispatch_handler, handler);
+REQUEST_STRING_ATTR_WRITER(request_set_dispatch_handler, handler);
 
 static VALUE request_request_time(VALUE self)
 {
     request_data *data;
 
     data = get_request_data(self);
+#ifdef APACHE2
+    return rb_time_new(apr_time_sec(data->request->request_time), 0);
+#else
     return rb_time_new(data->request->request_time, 0);
+#endif
 }
 
 static VALUE request_header_only(VALUE self)
@@ -547,7 +566,8 @@ static VALUE request_set_content_type(VALUE self, VALUE str)
 	Check_Type(str, T_STRING);
 	str = rb_funcall(str, rb_intern("downcase"), 0);
 	data->request->content_type =
-	    ap_pstrdup(data->request->pool, RSTRING(str)->ptr);
+	    ap_pstrndup(data->request->pool,
+			RSTRING(str)->ptr, RSTRING(str)->len);
     }
     return str;
 }
@@ -564,7 +584,9 @@ static VALUE request_set_content_encoding(VALUE self, VALUE str)
 	Check_Type(str, T_STRING);
 	str = rb_funcall(str, rb_intern("downcase"), 0);
 	data->request->content_encoding =
-	    ap_pstrdup(data->request->pool, RSTRING(str)->ptr);
+	    ap_pstrndup(data->request->pool,
+			RSTRING(str)->ptr,
+			RSTRING(str)->len);
     }
     return str;
 }
@@ -613,7 +635,9 @@ static VALUE request_set_content_languages(VALUE self, VALUE ary)
 	    VALUE str = RARRAY(ary)->ptr[i];
 	    str = rb_funcall(str, rb_intern("downcase"), 0);
 	    *(char **) ap_push_array(data->request->content_languages) =
-		ap_pstrdup(data->request->pool, STR2CSTR(str));
+		ap_pstrndup(data->request->pool,
+			    RSTRING(str)->ptr,
+			    RSTRING(str)->len);
 	}
     }
     return ary;
@@ -691,6 +715,64 @@ static VALUE request_notes(VALUE self)
     return data->notes;
 }
 
+#ifdef APACHE2
+static mode_t get_mode(apr_finfo_t *finfo)
+{
+    mode_t mode = 0;
+
+    if (finfo->protection & APR_UREAD)
+        mode |= S_IRUSR;
+    if (finfo->protection & APR_UWRITE)
+        mode |= S_IWUSR;
+    if (finfo->protection & APR_UEXECUTE)
+        mode |= S_IXUSR;
+
+    if (finfo->protection & APR_GREAD)
+        mode |= S_IRGRP;
+    if (finfo->protection & APR_GWRITE)
+        mode |= S_IWGRP;
+    if (finfo->protection & APR_GEXECUTE)
+        mode |= S_IXGRP;
+
+    if (finfo->protection & APR_WREAD)
+        mode |= S_IROTH;
+    if (finfo->protection & APR_WWRITE)
+        mode |= S_IWOTH;
+    if (finfo->protection & APR_WEXECUTE)
+        mode |= S_IXOTH;
+
+    switch (finfo->filetype) {
+    case APR_REG:
+	mode |= S_IFREG;
+	break;
+    case APR_DIR:
+	mode |= S_IFDIR;
+	break;
+    case APR_CHR:
+	mode |= S_IFCHR;
+	break;
+    case APR_BLK:
+	mode |= S_IFBLK;
+	break;
+    case APR_PIPE:
+	mode |= S_IFIFO;
+	break;
+    case APR_LNK:
+	mode |= S_IFLNK;
+	break;
+    case APR_SOCK:
+#if !defined(BEOS) && defined(S_ISSOCK)
+	mode |= S_IFSOCK;
+#endif
+	break;
+    default:
+	break;
+    }
+
+    return mode;
+}
+#endif /* APACHE2 */
+
 static VALUE request_finfo(VALUE self)
 {
     VALUE cStat;
@@ -701,7 +783,23 @@ static VALUE request_finfo(VALUE self)
     if (NIL_P(data->finfo)) {
 	cStat = rb_const_get(rb_cFile, rb_intern("Stat"));
 	data->finfo = Data_Make_Struct(cStat, struct stat, NULL, free, st);
+#ifdef APACHE2
+	memset(st, 0, sizeof(struct stat));
+	if (data->request->finfo.filetype != 0) {
+	    st->st_dev = data->request->finfo.device;
+	    st->st_ino = data->request->finfo.inode;
+	    st->st_nlink = data->request->finfo.nlink;
+	    st->st_mode = get_mode(&data->request->finfo);
+	    st->st_uid = data->request->finfo.user;
+	    st->st_gid = data->request->finfo.group;
+	    st->st_size = data->request->finfo.size;
+	    st->st_atime = apr_time_sec(data->request->finfo.atime);
+	    st->st_mtime = apr_time_sec(data->request->finfo.mtime);
+	    st->st_ctime = apr_time_sec(data->request->finfo.ctime);
+	}
+#else /* Apache 1.x */
 	*st = data->request->finfo;
+#endif
     }
     return data->finfo;
 }
@@ -744,7 +842,7 @@ static VALUE request_aset(VALUE self, VALUE key, VALUE val)
 static VALUE request_each_header(VALUE self)
 {
     request_data *data;
-    array_header *hdrs_arr;
+    const array_header *hdrs_arr;
     table_entry *hdrs;
     int i;
     VALUE assoc;
@@ -769,7 +867,7 @@ static VALUE request_each_header(VALUE self)
 static VALUE request_each_key(VALUE self)
 {
     request_data *data;
-    array_header *hdrs_arr;
+    const array_header *hdrs_arr;
     table_entry *hdrs;
     int i;
 
@@ -791,7 +889,7 @@ static VALUE request_each_key(VALUE self)
 static VALUE request_each_value(VALUE self)
 {
     request_data *data;
-    array_header *hdrs_arr;
+    const array_header *hdrs_arr;
     table_entry *hdrs;
     int i;
 
@@ -953,15 +1051,43 @@ static VALUE request_default_type(VALUE self)
     return type ? rb_str_new2(type) : Qnil;
 }
 
-static VALUE request_remote_host(VALUE self)
+static VALUE request_remote_host(int argc, VALUE *argv, VALUE self)
 {
     request_data *data;
     const char *host;
+    VALUE lookup_type;
+    int lookup_val = REMOTE_HOST;
+
+    if (argc == 1) {
+      rb_scan_args(argc, argv, "01", &lookup_type);
+      switch (NUM2INT(lookup_type)) {
+      case REMOTE_HOST:
+	lookup_val = REMOTE_HOST;
+	break;
+      case REMOTE_NAME:
+	lookup_val = REMOTE_NAME;
+	break;
+      case REMOTE_NOLOOKUP:
+	lookup_val = REMOTE_NOLOOKUP;
+	break;
+      case REMOTE_DOUBLE_REV:
+	lookup_val = REMOTE_DOUBLE_REV;
+	break;
+      default:
+	lookup_val = REMOTE_HOST;
+      }
+    }
 
     data = get_request_data(self);
+#ifdef APACHE2
     host = ap_get_remote_host(data->request->connection,
 			      data->request->per_dir_config,
-			      REMOTE_HOST);
+			      lookup_val, NULL);
+#else /* Apache 1.x */
+    host = ap_get_remote_host(data->request->connection,
+			      data->request->per_dir_config,
+			      lookup_val);
+#endif
     return host ? rb_str_new2(host) : Qnil;
 }
 
@@ -1001,6 +1127,8 @@ static VALUE request_server_port(VALUE self)
     return INT2NUM(ap_get_server_port(data->request));
 }
 
+#if 0
+
 static VALUE request_auth_type(VALUE self)
 {
     request_data *data;
@@ -1021,6 +1149,8 @@ static VALUE request_auth_name(VALUE self)
     return auth_name ? rb_str_new2(auth_name) : Qnil;
 }
 
+#endif
+
 static VALUE request_satisfies(VALUE self)
 {
     request_data *data;
@@ -1074,30 +1204,42 @@ static VALUE request_signature(VALUE self)
 
 static VALUE request_reset_timeout(VALUE self)
 {
+#ifdef APACHE2
+    rb_notimplement();
+#else /* Apache 1.x */
     request_data *data;
 
     data = get_request_data(self);
     ap_reset_timeout(data->request);
+#endif
     return Qnil;
 }
 
 static VALUE request_hard_timeout(VALUE self, VALUE name)
 {
+#ifdef APACHE2
+    rb_notimplement();
+#else /* Apache 1.x */
     request_data *data;
+    char *s;
 
+    Check_Type(name, T_STRING);
     data = get_request_data(self);
-    ap_hard_timeout(ap_pstrdup(data->request->pool, STR2CSTR(name)),
-		    data->request);
+    s = ap_pstrndup(data->request->pool, RSTRING(name)->ptr, RSTRING(name)->len);
+    ap_hard_timeout(s, data->request);
+#endif
     return Qnil;
 }
 
 static VALUE request_soft_timeout(VALUE self, VALUE name)
 {
     request_data *data;
+    char *s;
 
+    Check_Type(name, T_STRING);
     data = get_request_data(self);
-    ap_soft_timeout(ap_pstrdup(data->request->pool, STR2CSTR(name)),
-		    data->request);
+    s = ap_pstrndup(data->request->pool, RSTRING(name)->ptr, RSTRING(name)->len);
+    ap_soft_timeout(s, data->request);
     return Qnil;
 }
 
@@ -1185,17 +1327,25 @@ static VALUE request_setup_cgi_env(VALUE self)
 static VALUE request_log_reason(VALUE self, VALUE msg, VALUE file)
 {
     request_data *data;
+    const char *host;
 
     Check_Type(msg, T_STRING);
     Check_Type(file, T_STRING);
     data = get_request_data(self);
+#ifdef APACHE2
+    host = ap_get_remote_host(data->request->connection,
+			      data->request->per_dir_config,
+			      REMOTE_HOST, NULL);
+#else /* Apache 1.x */
+    host = ap_get_remote_host(data->request->connection,
+			      data->request->per_dir_config,
+			      REMOTE_HOST);
+#endif
     ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO,
-		 data->request->server,
+		 APLOG_STATUS(0) data->request->server,
 		 "access to %s failed for %s, reason: %s",
 		 STR2CSTR(file),
-		 ap_get_remote_host(data->request->connection,
-				    data->request->per_dir_config,
-				    REMOTE_NAME),
+		 host,
 		 STR2CSTR(msg));
     return Qnil;
 }
@@ -1225,6 +1375,319 @@ static VALUE request_exception(VALUE self)
     return data->exception;
 }
 
+#ifdef APACHE2
+REQUEST_STRING_ATTR_READER(request_user, user);
+
+static VALUE request_set_user(VALUE self, VALUE val)
+{
+    request_data *data;
+
+    Check_Type(val, T_STRING);
+    data = get_request_data(self);
+    data->request->user = ap_pstrndup(data->request->pool,
+				      RSTRING(val)->ptr,
+				      RSTRING(val)->len);
+    return val;
+}
+#else /* Apache 1.x */
+static VALUE request_user(VALUE self)
+{
+    VALUE conn;
+
+    conn = request_connection(self);
+    return rb_funcall(conn, rb_intern("user"), 0);
+}
+
+static VALUE request_set_user(VALUE self, VALUE val)
+{
+    VALUE conn;
+
+    conn = request_connection(self);
+    return rb_funcall(conn, rb_intern("user="), 1, val);
+}
+#endif
+
+/* Should only be called from inside of response handlers */
+static VALUE request_internal_redirect(VALUE self, VALUE uri)
+{
+    request_data *data;
+
+    Check_Type(uri, T_STRING);
+    data = get_request_data(self);
+    ap_internal_redirect(STR2CSTR(uri), data->request);
+    return Qnil;
+}
+
+static VALUE request_custom_response(VALUE self, VALUE status, VALUE string)
+{
+    request_data *data;
+
+    Check_Type(status, T_FIXNUM);
+    Check_Type(string, T_STRING);
+    data = get_request_data(self);
+    ap_custom_response(data->request, NUM2INT(status), STR2CSTR(string));
+    return Qnil;
+}
+
+static VALUE request_is_initial(VALUE self)
+{
+    request_data *data;
+
+    data = get_request_data(self);
+    if (ap_is_initial_req(data->request)) {
+        return Qtrue;
+    } else {
+        return Qfalse;
+    }
+}
+
+static VALUE request_is_main(VALUE self)
+{
+    request_data *data;
+
+    data = get_request_data(self);
+    if (data->request->main) {
+        return Qfalse;
+    } else {
+        return Qtrue;
+    }
+}
+
+static VALUE request_auth_type(VALUE self)
+{
+    request_data *data;
+    core_dir_config *conf;
+
+    data = get_request_data(self);
+    conf = (core_dir_config *)
+	ap_get_module_config(data->request->per_dir_config, &core_module);
+    if (conf->ap_auth_type) {
+	return rb_tainted_str_new2(conf->ap_auth_type);
+    }
+    else {
+	return Qnil;
+    }
+}
+
+static VALUE request_set_auth_type(VALUE self, VALUE val)
+{
+    request_data *data;
+    core_dir_config *conf;
+
+    Check_Type(val, T_STRING);
+    data = get_request_data(self);
+    conf = (core_dir_config *)
+	ap_get_module_config(data->request->per_dir_config, &core_module);
+    conf->ap_auth_type = ap_pstrndup(data->request->pool,
+				     RSTRING(val)->ptr,
+				     RSTRING(val)->len);
+    ap_set_module_config(data->request->per_dir_config, &core_module, conf);
+    return val;
+}
+
+static VALUE request_auth_name(VALUE self)
+{
+    request_data *data;
+    core_dir_config *conf;
+
+    data = get_request_data(self);
+    conf = (core_dir_config *)
+	ap_get_module_config(data->request->per_dir_config, &core_module);
+    if (conf->ap_auth_name) {
+	return rb_tainted_str_new2(conf->ap_auth_name);
+    }
+    else {
+	return Qnil;
+    }
+}
+
+static VALUE request_set_auth_name(VALUE self, VALUE val)
+{
+    request_data *data;
+    core_dir_config *conf;
+
+    Check_Type(val, T_STRING);
+    data = get_request_data(self);
+    conf = (core_dir_config *)
+	ap_get_module_config(data->request->per_dir_config, &core_module);
+    conf->ap_auth_name = ap_pstrndup(data->request->pool,
+				     RSTRING(val)->ptr,
+				     RSTRING(val)->len);
+    ap_set_module_config(data->request->per_dir_config, &core_module, conf);
+    return val;
+}
+
+static VALUE request_default_charset(VALUE self)
+{
+    request_data *data = get_request_data(self);
+    core_dir_config *conf =
+	ap_get_module_config(data->request->per_dir_config, &core_module);
+    const char *charset_name =
+	conf->add_default_charset_name;
+    return charset_name ? rb_tainted_str_new2(charset_name) : Qnil;
+}
+
+static VALUE request_bytes_sent(VALUE self)
+{
+    request_data *data;
+    request_rec *last;
+
+    data = get_request_data(self);
+
+    for (last = data->request; last->next; last = data->request->next)
+        continue;
+
+#ifndef APACHE2
+    if (last->sent_bodyct && !last->bytes_sent)
+        ap_bgetopt(last->connection->client, BO_BYTECT, &last->bytes_sent);
+#endif
+
+    return INT2NUM(last->bytes_sent);
+}
+
+static VALUE request_send_fd(VALUE self, VALUE io)
+{
+    OpenFile *fptr;
+#ifdef APACHE2
+    apr_size_t bytes_sent;
+#else
+    long bytes_sent;
+#endif
+    request_data *data;
+
+    request_set_sync(self, Qtrue);
+    rb_apache_request_flush(self);    
+    data = get_request_data(self);
+
+    GetOpenFile(io, fptr);
+
+#ifdef APACHE2
+    ap_send_fd((apr_file_t *)fptr->f, data->request, 0, -1, &bytes_sent);
+#else
+    bytes_sent = ap_send_fd_length(fptr->f, data->request, -1);
+#endif
+
+    return INT2NUM(bytes_sent);
+}
+
+static VALUE request_proxy_q(VALUE self)
+{
+    request_data *data = get_request_data(self);
+
+    switch (data->request->proxyreq) {
+#ifdef APACHE2
+    case PROXYREQ_NONE:
+	return Qfalse;
+    case PROXYREQ_PROXY:
+    case PROXYREQ_REVERSE:
+	return Qtrue;
+#else
+    case NOT_PROXY:
+	return Qfalse;
+    case STD_PROXY:
+    case PROXY_PASS:
+	return Qtrue;
+#endif
+    default:
+      rb_raise(rb_eArgError, "Unknown Proxy Type");
+    }
+}
+
+static VALUE request_proxy_pass_q(VALUE self)
+{
+    request_data *data = get_request_data(self);
+
+#ifdef APACHE2
+    if (data->request->proxyreq == PROXYREQ_REVERSE)
+#else
+    if (data->request->proxyreq == PROXY_PASS)
+#endif
+      return Qtrue;
+    else
+      return Qfalse;
+}
+
+static VALUE request_get_cache_resp(VALUE self)
+{
+    request_data *data;
+    table *tbl;
+    table_entry *hdrs;
+    const array_header *hdrs_arr;
+    int i;
+
+    data = get_request_data(self);
+
+    if (NIL_P(data->headers_out))
+        data->headers_out = rb_apache_table_new(rb_cApacheTable,
+					       data->request->headers_out);
+
+    Data_Get_Struct(data->headers_out, table, tbl);
+    hdrs_arr = ap_table_elts(tbl);
+    hdrs = (table_entry *) hdrs_arr->elts;
+    for (i = 0; i < hdrs_arr->nelts; i++) {
+        if (hdrs[i].key == NULL)
+	    continue;
+	if (strcasecmp(hdrs[i].key, "Pragma") ||
+	    strcasecmp(hdrs[i].key, "Cache-control")) {
+	    return Qtrue;
+	}
+    }
+    return Qfalse;
+}
+
+static VALUE request_set_cache_resp(VALUE self, VALUE arg)
+{
+    request_data *data;
+    table *tbl;
+
+    data = get_request_data(self);
+
+    if (NIL_P(data->headers_out))
+        data->headers_out = rb_apache_table_new(rb_cApacheTable,
+					       data->request->headers_out);
+
+    Data_Get_Struct(data->headers_out, table, tbl);
+    if (arg == Qtrue) {
+        ap_table_setn(tbl, "Pragma", "no-cache");
+	ap_table_setn(tbl, "Cache-control", "no-cache");
+	return Qtrue;
+    } else {
+        ap_table_unset(tbl, "Pragma");
+	ap_table_unset(tbl, "Cache-control");
+	return Qfalse;
+    }
+}
+
+static VALUE request_lookup_uri(VALUE self, VALUE uri)
+{
+    request_data *data;
+    request_rec *new_rec;
+
+    Check_Type(uri, T_STRING);
+    data = get_request_data(self);
+#ifdef APACHE2
+    new_rec = ap_sub_req_lookup_uri(STR2CSTR(uri), data->request, NULL);
+#else
+    new_rec = ap_sub_req_lookup_uri(STR2CSTR(uri), data->request);
+#endif
+    return apache_request_new(new_rec);
+}
+
+static VALUE request_lookup_file(VALUE self, VALUE file)
+{
+    request_data *data;
+    request_rec *new_rec;
+
+    Check_Type(file, T_STRING);
+    data = get_request_data(self);
+#ifdef APACHE2
+    new_rec = ap_sub_req_lookup_file(STR2CSTR(file), data->request, NULL);
+#else
+    new_rec = ap_sub_req_lookup_file(STR2CSTR(file), data->request);
+#endif
+    return apache_request_new(new_rec);
+}
+
 void rb_init_apache_request()
 {
     rb_eApachePrematureChunkEndError =
@@ -1264,6 +1727,7 @@ void rb_init_apache_request()
     rb_define_method(rb_cApacheRequest, "server", request_server, 0);
     rb_define_method(rb_cApacheRequest, "next", request_next, 0);
     rb_define_method(rb_cApacheRequest, "prev", request_prev, 0);
+    rb_define_method(rb_cApacheRequest, "last", request_last, 0);
     rb_define_method(rb_cApacheRequest, "main", request_main, 0);
     rb_define_method(rb_cApacheRequest, "protocol", request_protocol, 0);
     rb_define_method(rb_cApacheRequest, "hostname", request_hostname, 0);
@@ -1286,10 +1750,13 @@ void rb_init_apache_request()
 		     request_request_method, 0);
     rb_define_method(rb_cApacheRequest, "method_number",
 		     request_method_number, 0);
+    rb_define_method(rb_cApacheRequest, "allowed", request_get_allowed, 0);
+    rb_define_method(rb_cApacheRequest, "allowed=", request_set_allowed, 1);
     rb_define_method(rb_cApacheRequest, "the_request",
 		     request_the_request, 0);
     rb_define_method(rb_cApacheRequest, "header_only?", request_header_only, 0);
-    rb_define_method(rb_cApacheRequest, "args", request_args, 0);
+    rb_define_method(rb_cApacheRequest, "args", request_get_args, 0);
+    rb_define_method(rb_cApacheRequest, "args=", request_set_args, 1);
     rb_define_method(rb_cApacheRequest, "content_length",
 		     request_content_length, 0);
     rb_define_method(rb_cApacheRequest, "content_type",
@@ -1336,15 +1803,14 @@ void rb_init_apache_request()
 		     request_allow_overrides, 0);
     rb_define_method(rb_cApacheRequest, "default_type",
 		     request_default_type, 0);
-    rb_define_method(rb_cApacheRequest, "remote_host", request_remote_host, 0);
+    rb_define_method(rb_cApacheRequest, "remote_host",
+		     request_remote_host, -1);
     rb_define_method(rb_cApacheRequest, "remote_logname",
 		     request_remote_logname, 0);
     rb_define_method(rb_cApacheRequest, "construct_url",
 		     request_construct_url, 1);
     rb_define_method(rb_cApacheRequest, "server_name", request_server_name, 0);
     rb_define_method(rb_cApacheRequest, "server_port", request_server_port, 0);
-    rb_define_method(rb_cApacheRequest, "auth_type", request_auth_type, 0);
-    rb_define_method(rb_cApacheRequest, "auth_name", request_auth_name, 0);
     rb_define_method(rb_cApacheRequest, "satisfies", request_satisfies, 0);
     rb_define_method(rb_cApacheRequest, "requires", request_requires, 0);
     rb_define_method(rb_cApacheRequest, "escape_html", request_escape_html, 1);
@@ -1357,6 +1823,14 @@ void rb_init_apache_request()
 		     request_soft_timeout, 1);
     rb_define_method(rb_cApacheRequest, "kill_timeout",
 		     request_kill_timeout, 0);
+    rb_define_method(rb_cApacheRequest, "internal_redirect",
+		     request_internal_redirect, 1);
+    rb_define_method(rb_cApacheRequest, "custom_response",
+		     request_custom_response, 2);
+    rb_define_method(rb_cApacheRequest, "main?",
+		     request_is_main, 0);
+    rb_define_method(rb_cApacheRequest, "initial?",
+		     request_is_initial, 0);
     rb_define_method(rb_cApacheRequest, "note_auth_failure",
 		     request_note_auth_failure, 0);
     rb_define_method(rb_cApacheRequest, "note_basic_auth_failure",
@@ -1377,4 +1851,30 @@ void rb_init_apache_request()
 		     request_error_message, 0);
     rb_define_method(rb_cApacheRequest, "exception",
 		     request_exception, 0);
+    rb_define_method(rb_cApacheRequest, "user",
+		     request_user, 0);
+    rb_define_method(rb_cApacheRequest, "user=",
+		     request_set_user, 1);
+    rb_define_method(rb_cApacheRequest, "auth_type", request_auth_type, 0);
+    rb_define_method(rb_cApacheRequest, "auth_type=", request_set_auth_type, 1);
+    rb_define_method(rb_cApacheRequest, "auth_name", request_auth_name, 0);
+    rb_define_method(rb_cApacheRequest, "auth_name=", request_set_auth_name, 1);
+    rb_define_method(rb_cApacheRequest, "default_charset",
+		     request_default_charset, 0);
+    rb_define_method(rb_cApacheRequest, "bytes_sent", request_bytes_sent, 0);
+    rb_define_method(rb_cApacheRequest, "send_fd", request_send_fd, 1);
+    rb_define_method(rb_cApacheRequest, "proxy?",
+		     request_proxy_q, 0);
+    rb_define_method(rb_cApacheRequest, "proxy_pass?",
+		     request_proxy_pass_q, 0);
+    rb_define_method(rb_cApacheRequest, "dispatch_handler",
+		     request_get_dispatch_handler, 0);
+    rb_define_method(rb_cApacheRequest, "dispatch_handler=",
+		     request_set_dispatch_handler, 1);
+    rb_define_method(rb_cApacheRequest, "cache_resp",
+		     request_get_cache_resp, 0);
+    rb_define_method(rb_cApacheRequest, "cache_resp=",
+		     request_set_cache_resp, 1);
+    rb_define_method(rb_cApacheRequest, "lookup_uri", request_lookup_uri, 1);
+    rb_define_method(rb_cApacheRequest, "lookup_file", request_lookup_file, 1);
 }
