@@ -1,5 +1,5 @@
 /*
- * $Id: ruby_config.c,v 1.22 2002/01/30 08:42:34 shugo Exp $
+ * $Id: ruby_config.c,v 1.29 2002/08/08 04:13:28 shugo Exp $
  * Copyright (C) 2000  ZetaBITS, Inc.
  * Copyright (C) 2000  Information-technology Promotion Agency, Japan
  * Copyright (C) 2001  Shugo Maeda <shugo@modruby.net>
@@ -27,21 +27,11 @@
  * SUCH DAMAGE.
  */
 
-#include "httpd.h"
-#include "http_config.h"
-#include "http_core.h"
-#include "http_log.h"
-#include "http_main.h"
-#include "http_protocol.h"
-#include "http_request.h"
-#include "util_script.h"
-#include "multithread.h"
-
-#include "ruby.h"
-
 #include "mod_ruby.h"
 #include "ruby_config.h"
 
+static int default_safe_level = MR_DEFAULT_SAFE_LEVEL;
+
 #define push_handler(p, handler, arg) { \
     if ((handler) == NULL) \
 	(handler) = ap_make_array(p, 1, sizeof(char*)); \
@@ -59,6 +49,27 @@ void *ruby_create_server_config(pool *p, server_rec *s)
     return conf;
 }
 
+void *ruby_merge_server_config(pool *p, void *basev, void *addv)
+{
+    ruby_server_config *new =
+	(ruby_server_config *) ap_pcalloc(p, sizeof(ruby_server_config));
+    ruby_server_config *base = (ruby_server_config *) basev;
+    ruby_server_config *add = (ruby_server_config *) addv;
+
+    if (add->load_path == NULL) {
+	new->load_path = base->load_path;
+    }
+    else if (base->load_path == NULL) {
+	new->load_path = add->load_path;
+    }
+    else {
+	new->load_path = ap_append_arrays(p, base->load_path, add->load_path);
+    }
+    new->env = ap_overlay_tables(p, add->env, base->env);
+    new->timeout = add->timeout ? add->timeout : base->timeout;
+    return (void *) new;
+}
+
 void *ruby_create_dir_config(pool *p, char *dirname)
 {
     ruby_dir_config *conf =
@@ -66,8 +77,9 @@ void *ruby_create_dir_config(pool *p, char *dirname)
 
     conf->kcode = NULL;
     conf->env = ap_make_table(p, 5); 
-    conf->safe_level = MR_DEFAULT_SAFE_LEVEL;
+    conf->safe_level = default_safe_level;
     conf->output_mode = MR_OUTPUT_DEFAULT;
+    conf->load_path = NULL;
     conf->ruby_handler = NULL;
     conf->ruby_trans_handler = NULL;
     conf->ruby_authen_handler = NULL;
@@ -112,6 +124,16 @@ void *ruby_merge_dir_config(pool *p, void *basev, void *addv)
     }
     new->output_mode = add->output_mode ? add->output_mode : base->output_mode;
 
+    if (add->load_path == NULL) {
+	new->load_path = base->load_path;
+    }
+    else if (base->load_path == NULL) {
+	new->load_path = add->load_path;
+    }
+    else {
+	new->load_path = ap_append_arrays(p, base->load_path, add->load_path);
+    }
+
     new->ruby_handler =
 	merge_handlers(p, base->ruby_handler, add->ruby_handler);
     new->ruby_trans_handler =
@@ -147,34 +169,44 @@ const char *ruby_cmd_kanji_code(cmd_parms *cmd, ruby_dir_config *conf, char *arg
     return NULL;
 }
 
-const char *ruby_cmd_add_path(cmd_parms *cmd, void *dummy, char *arg)
+const char *ruby_cmd_add_path(cmd_parms *cmd, ruby_dir_config *dconf, char *arg)
 {
-    ruby_server_config *conf = get_server_config(cmd->server);
+    ruby_server_config *sconf;
 
-    if (ruby_running()) {
-	ruby_add_path(arg);
+    if (cmd->path == NULL) {
+	sconf = get_server_config(cmd->server);
+	*(char **) ap_push_array(sconf->load_path) = arg;
     }
     else {
-	*(char **) ap_push_array(conf->load_path) = arg;
+	if (dconf->load_path == NULL)
+	    dconf->load_path = ap_make_array(cmd->pool, 1, sizeof(char*));
+	*(char **) ap_push_array(dconf->load_path) = arg;
     }
     return NULL;
 }
 
-const char *ruby_cmd_require(cmd_parms *cmd, void *dummy, char *arg)
+const char *ruby_cmd_require(cmd_parms *cmd, ruby_dir_config *dconf, char *arg)
 {
+    ruby_server_config *sconf = get_server_config(cmd->server);
+    ruby_library_context *lib;
     int state;
 
     if (ruby_running()) {
-	if ((state = ruby_require(arg))) {
-	    ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO, cmd->server,
+	if ((state = ruby_require(arg, sconf, dconf))) {
+	    ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO,
+			 APLOG_STATUS(0) cmd->server,
 			 "mod_ruby: failed to require %s", arg);
 	    ruby_log_error(cmd->server, ruby_get_error_info(state));
 	}
     }
     else {
 	if (ruby_required_libraries == NULL)
-	    ruby_required_libraries = ap_make_array(cmd->pool, 1, sizeof(char*));
-	*(char **) ap_push_array(ruby_required_libraries) = arg;
+	    ruby_required_libraries =
+		ap_make_array(cmd->pool, 1, sizeof(ruby_library_context));
+	lib = (ruby_library_context *) ap_push_array(ruby_required_libraries);
+	lib->filename = arg;
+	lib->server_config = sconf;
+	lib->dir_config = dconf;
     }
     return NULL;
 }
@@ -218,7 +250,12 @@ const char *ruby_cmd_timeout(cmd_parms *cmd, void *dummy, char *arg)
 
 const char *ruby_cmd_safe_level(cmd_parms *cmd, ruby_dir_config *conf, char *arg)
 {
-    conf->safe_level = atoi(arg);
+    if (cmd->path == NULL && !cmd->server->is_virtual) {
+	conf->safe_level = default_safe_level = atoi(arg);
+    }
+    else {
+	conf->safe_level = atoi(arg);
+    }
     return NULL;
 }
 
